package net.knarcraft.blacksmith.config.blacksmith;

import net.knarcraft.blacksmith.config.Setting;
import net.knarcraft.blacksmith.config.SettingValueType;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * An enum representing all of Blacksmith's settings
 */
public enum BlacksmithSetting implements Setting {

    /**
     * The setting for whether the NPC should drop an item to the ground when finished
     *
     * <p>If set to false, the item will be directly put in the player's inventory instead</p>
     */
    DROP_ITEM("dropItem", SettingValueType.BOOLEAN, true, "dropItem", true, false),

    /**
     * The setting for the chance of a reforging to fail
     */
    FAIL_CHANCE("failReforgeChance", SettingValueType.PERCENTAGE, 10, "failReforgeChance",
            true, false),

    /**
     * The setting for whether failing a reforging should downgrade/remove enchantments as well
     */
    FAIL_REMOVE_ENCHANTMENTS("failReforgeRemovesEnchantments", SettingValueType.BOOLEAN, false,
            "failReforgeRemovesEnchantments", true, false),

    /**
     * The setting for the chance of an additional enchantment being added
     */
    EXTRA_ENCHANTMENT_CHANCE("extraEnchantmentChance", SettingValueType.PERCENTAGE, 5,
            "extraEnchantmentChance", true, false),

    /**
     * The setting for the maximum amount of enchantments that can be added to an item
     */
    MAX_ENCHANTMENTS("maxEnchantments", SettingValueType.POSITIVE_INTEGER, 3,
            "maxEnchantments", true, false),

    /**
     * The maximum amount of seconds a player may need to wait for the reforging to finish
     */
    MAX_REFORGE_DELAY("delaysInSeconds.maximum", SettingValueType.POSITIVE_INTEGER, 30,
            "maxReforgeDelay", true, false),

    /**
     * The minimum amount of seconds a player may need to wait for the reforging to finish
     */
    MIN_REFORGE_DELAY("delaysInSeconds.minimum", SettingValueType.POSITIVE_INTEGER, 5,
            "minReforgeDelay", true, false),

    /**
     * The setting for number of seconds a player has to wait between each usage of the blacksmith
     */
    REFORGE_COOL_DOWN("delaysInSeconds.reforgeCoolDown", SettingValueType.POSITIVE_INTEGER, 60,
            "reforgeCoolDown", true, false),

    /**
     * The setting for which items the blacksmith is able to reforge
     */
    REFORGE_ABLE_ITEMS("reforgeAbleItems", SettingValueType.REFORGE_ABLE_ITEMS, "",
            "reforgeAbleItems", true, false),

    /**
     * The setting for the title used to display which kind of blacksmith the NPC is
     *
     * <p>While this should be entirely configurable, values such as armor-smith, sword-smith and similar, which
     * describe the blacksmith's specialization, and thus the range of reforge-able items, is expected.</p>
     */
    BLACKSMITH_TITLE("blacksmithTitle", SettingValueType.STRING, "blacksmith",
            "blacksmithTitle", true, false),

    /**
     * The setting for the enchantments a blacksmith cannot apply to items
     */
    ENCHANTMENT_BLOCKLIST("enchantmentBlocklist", SettingValueType.STRING_LIST, new String[]{"binding_curse",
            "mending", "vanishing_curse"}, "enchantmentBlocklist", true, false),

    /**
     * Whether to allow this blacksmith to repair anvils
     */
    REPAIR_ANVILS("reforgeAnvils", SettingValueType.BOOLEAN, false, "reforgeAnvils",
            true, false),

    /*-----------
     | Messages |
     -----------*/

    /**
     * The message displayed when the blacksmith is busy with another player
     */
    BUSY_WITH_PLAYER_MESSAGE("messages.busyPlayerMessage", SettingValueType.STRING,
            "&cI'm busy at the moment. Come back later!", "busyPlayerMessage",
            true, true),

    /**
     * The message displayed when the blacksmith is already reforging something for the player
     */
    BUSY_WITH_REFORGE_MESSAGE("messages.busyReforgeMessage", SettingValueType.STRING,
            "&cI'm working on it. Be patient! I'll finish {time}!", "busyReforgeMessage",
            true, true),

    /**
     * The message displayed if the player has to wait for the cool-down to expire
     */
    COOL_DOWN_UNEXPIRED_MESSAGE("messages.coolDownUnexpiredMessage", SettingValueType.STRING,
            "&cYou've already had your chance! Give me a break! I'll be ready {time}!",
            "coolDownUnexpiredMessage", true, true),

    /**
     * The message displayed when displaying the cost of reforging the held item to the player
     */
    COST_MESSAGE("messages.costMessage", SettingValueType.STRING,
            "&eIt will cost &a{cost}&e to reforge that &a{item}&e! Click again to reforge!",
            "costMessage", true, true),

    /**
     * The message displayed if the blacksmith fails reforging an item
     */
    FAIL_MESSAGE("messages.failReforgeMessage", SettingValueType.STRING,
            "&cWhoops! Didn't mean to do that! Maybe next time?", "failReforgeMessage",
            true, true),

    /**
     * The message displayed if a player is unable to pay the blacksmith
     */
    INSUFFICIENT_FUNDS_MESSAGE("messages.insufficientFundsMessage", SettingValueType.STRING,
            "&cYou don't have enough money to reforge that item!", "insufficientFundsMessage",
            true, true),

    /**
     * The message displayed if the blacksmith encounters an item they cannot reforge
     */
    INVALID_ITEM_MESSAGE("messages.invalidItemMessage", SettingValueType.STRING,
            "&cI'm sorry, but I'm a/an {title}, I don't know how to reforge that!",
            "invalidItemMessage", true, true),

    /**
     * The message displayed if a player presents a different item after seeing the price to reforge an item
     */
    ITEM_UNEXPECTEDLY_CHANGED_MESSAGE("messages.itemChangedMessage", SettingValueType.STRING,
            "&cThat's not the item you wanted to reforge before!", "itemChangedMessage",
            true, true),

    /**
     * The message displayed when the blacksmith starts reforging an item
     */
    START_REFORGE_MESSAGE("messages.startReforgeMessage", SettingValueType.STRING,
            "&eOk, let's see what I can do...", "startReforgeMessage", true, true),

    /**
     * The message displayed when the blacksmith successfully finishes reforging an item
     */
    SUCCESS_MESSAGE("messages.successMessage", SettingValueType.STRING,
            "There you go! All better!", "successMessage", true, true),

    /**
     * The message displayed when trying to reforge an item with full durability
     */
    NOT_DAMAGED_MESSAGE("messages.notDamagedMessage", SettingValueType.STRING,
            "&cThat item is not in need of repair", "notDamagedMessage", true, true),

    /*------------------
     | Global settings |
     ------------------*/

    /**
     * The base price for repairing, regardless of durability
     *
     * <p>This allows specifying a price for each item, by setting basePrice.item_name.</p>
     */
    BASE_PRICE("basePrice.default", SettingValueType.POSITIVE_DOUBLE, 10.0, "basePrice",
            false, false),

    /**
     * The base price for each durability point
     *
     * <p>If natural cost, this is the cost each missing durability point will add to the cost. If not natural cost,
     * this is the cost each present durability point will add to the cost. This allows specifying a price per
     * durability point value for each item, by setting pricePerDurabilityPoint.item_name</p>
     */
    PRICE_PER_DURABILITY_POINT("pricePerDurabilityPoint.default", SettingValueType.POSITIVE_DOUBLE,
            0.005, "pricePerDurabilityPoint", false, false),

    /**
     * The price increase for each level of each present enchantment
     *
     * <p>This can be specified for each possible enchantment by setting enchantment-cost.enchantment_name</p>
     */
    ENCHANTMENT_COST("enchantmentCost.default", SettingValueType.POSITIVE_DOUBLE, 5.0,
            "enchantmentCost", false, false),

    /**
     * Whether the cost should increase for damage taken, as opposed to increase for durability present
     */
    NATURAL_COST("useNaturalCost", SettingValueType.BOOLEAN, true, "useNaturalCost",
            false, false),

    /**
     * Whether to show exact time when displaying the wait time for a reforging or the cool-down
     */
    SHOW_EXACT_TIME("showExactTime", SettingValueType.BOOLEAN, false, "showExactTime",
            false, false),

    /**
     * The cost for repairing a chipped anvil
     */
    ANVIL_CHIPPED_COST("chippedAnvilReforgingCost", SettingValueType.POSITIVE_DOUBLE, 10.0,
            "chippedAnvilReforgingCost", false, false),

    /**
     * The cost for repairing a damaged anvil
     */
    ANVIL_DAMAGED_COST("damagedAnvilReforgingCost", SettingValueType.POSITIVE_DOUBLE, 20.0,
            "damagedAnvilReforgingCost", false, false);

    private final String path;
    private final String childPath;
    private final Object value;
    private final String commandName;
    private final SettingValueType valueType;
    private final boolean isPerNPC;
    private final boolean isMessage;

    /**
     * Instantiates a new setting
     *
     * @param path        <p>The full config path for this setting</p>
     * @param valueType   <p>The type of value used by this setting</p>
     * @param value       <p>The default value of this setting</p>
     * @param commandName <p>The name of the command used to change this setting</p>
     * @param isPerNPC    <p>Whether this setting is per-NPC or global</p>
     * @param isMessage   <p>Whether this option is for an NPC message</p>
     */
    BlacksmithSetting(String path, SettingValueType valueType, Object value, String commandName, boolean isPerNPC,
                      boolean isMessage) {
        if (isPerNPC) {
            this.path = "blacksmith.defaults." + path;
        } else {
            this.path = "blacksmith.global." + path;
        }
        this.value = value;
        this.valueType = valueType;
        this.childPath = path;
        this.commandName = commandName;
        this.isPerNPC = isPerNPC;
        this.isMessage = isMessage;
    }

    @Override
    public @NotNull String getPath() {
        return path;
    }

    @Override
    public @NotNull String getChildPath() {
        return childPath;
    }

    @Override
    public @NotNull Object getDefaultValue() {
        return value;
    }

    @Override
    public @NotNull String getCommandName() {
        return commandName;
    }

    @Override
    public @NotNull SettingValueType getValueType() {
        return this.valueType;
    }

    @Override
    public boolean isPerNPC() {
        return this.isPerNPC;
    }

    @Override
    public boolean isMessage() {
        return this.isMessage;
    }

    /**
     * Gets the blacksmith setting specified by the input string
     *
     * @param input <p>The input to check</p>
     * @return <p>The matching blacksmith setting, or null if not found</p>
     */
    public static @Nullable BlacksmithSetting getSetting(@NotNull String input) {
        for (BlacksmithSetting blacksmithSetting : BlacksmithSetting.values()) {
            if (input.equalsIgnoreCase(blacksmithSetting.commandName)) {
                return blacksmithSetting;
            }
        }
        return null;
    }

}
